//+------------------------------------------------------------------+ 
//|                                                     T3 bands.mq4 | 
//|                                                                  |
//| T3 deviation calculation originally developed                    |
//| by Bob Fulks and Alex Matulich                                   |
//+------------------------------------------------------------------+ 
#property copyright "mladen"
#property link      "mladenfx@gmail.com" 


#property indicator_separate_window
#property indicator_buffers 1
#property indicator_color1  DeepSkyBlue
#property indicator_width1  2
//
//
//
//
//

extern string TimeFrame   = "Current time frame";
extern int    T3Period    = 14;
extern double T3Hot       = 0.7;
extern int    T3Price     = PRICE_CLOSE; 
extern bool   T3Original  = false;
extern bool   Interpolate = true;

//
//
//
//
//

double devs[];

//
//
//
//
//

int    timeFrame;
string indicatorFileName;
bool   returnBars;
bool   calculateValue;

double c1;
double c2;
double c3;
double c4;

//+------------------------------------------------------------------+  
//|                                                                  |
//+------------------------------------------------------------------+ 
//
//
//
//
//

int init()
{
   SetIndexBuffer(0,devs);

      T3Hot = MathMax(MathMin(T3Hot,1),0.0001);
      double a  = T3Hot;
             c1 = -a*a*a;
             c2 =  3*(a*a+a*a*a);
             c3 = -3*(2*a*a+a+a*a*a);
             c4 = 1+3*a+a*a*a+3*a*a;

      //
      //
      //
      //
      //
         
      indicatorFileName = WindowExpertName();
      calculateValue    = (TimeFrame=="calculateValue"); if (calculateValue) return(0);
      returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);

   //
   //
   //
   //
   //
      
   IndicatorShortName(timeFrameToString(timeFrame)+" T3 deviation ("+T3Period+")");
   return(0);
}
//+------------------------------------------------------------------+  
//|                                                                  |
//+------------------------------------------------------------------+ 
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { devs[0] = limit+1; return(0); }

   //
   //
   //
   //
   //

   if (calculateValue || timeFrame==Period())
   {
      for(i=limit; i>=0; i--) devs[i] = iT3Deviation(iMA(NULL,0,1,0,MODE_SMA,T3Price,i),T3Period,i);
      return(0);
   }
   
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         devs[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",T3Period,T3Hot,T3Price,T3Original,0,y);

         //
         //
         //
         //
         //
      
         if (timeFrame <= Period() || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;
         if (!Interpolate) continue;

         //
         //
         //
         //
         //

         datetime time = iTime(NULL,timeFrame,y);
            for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            for(int k = 1; k < n; k++)
               devs[i+k] = devs[i] +(devs[i+n]-devs[i])*k/n;
   }

   //
   //
   //
   //
   //
   
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double t3Values[][36];
double iT3(double price,int period, int i,int s)
{
   if (ArrayRange(t3Values,0) != Bars) ArrayResize(t3Values,Bars);

   //
   //
   //
   //
   //
   
   int    r = Bars-i-1;
   double n = period; if (!T3Original) n = 1.0 + (n-1.0)/2.0;
   double alpha = 2.0/(1.0 + n);
   
   if (r < 2)
      {
         t3Values[r][s+0] = price;
         t3Values[r][s+1] = price;
         t3Values[r][s+2] = price;
         t3Values[r][s+3] = price;
         t3Values[r][s+4] = price;
         t3Values[r][s+5] = price;
      }
   else
      {
         t3Values[r][s+0] = t3Values[r-1][s+0]+alpha*(price           -t3Values[r-1][s+0]);
         t3Values[r][s+1] = t3Values[r-1][s+1]+alpha*(t3Values[r][s+0]-t3Values[r-1][s+1]);
         t3Values[r][s+2] = t3Values[r-1][s+2]+alpha*(t3Values[r][s+1]-t3Values[r-1][s+2]);
         t3Values[r][s+3] = t3Values[r-1][s+3]+alpha*(t3Values[r][s+2]-t3Values[r-1][s+3]);
         t3Values[r][s+4] = t3Values[r-1][s+4]+alpha*(t3Values[r][s+3]-t3Values[r-1][s+4]);
         t3Values[r][s+5] = t3Values[r-1][s+5]+alpha*(t3Values[r][s+4]-t3Values[r-1][s+5]);
      }
   return(c1*t3Values[r][s+5] + c2*t3Values[r][s+4] + c3*t3Values[r][s+3] + c4*t3Values[r][s+2]);
}

//
//
//
//
//

double t3Valued[][6];
double iT3Deviation(double price, int period, int i)
{
   if (ArrayRange(t3Valued,0) != Bars) ArrayResize(t3Valued,Bars);
   
   //
   //
   //
   //
   //
   
   int    r = Bars-i-1;
   double n = period; if (!T3Original) n = 1.0 + (n-1)/2.0;
   double alpha = 2.0/(1.0 + n);

   double temp = iT3(price         ,period,i, 0); t3Valued[r][0] = MathSqrt(alpha*(price         -temp)*(price         -temp)+(1.0-alpha)*t3Valued[r-1][0]*t3Valued[r-1][0]);
          temp = iT3(t3Valued[r][0],period,i, 6); t3Valued[r][1] = MathSqrt(alpha*(t3Valued[r][0]-temp)*(t3Valued[r][0]-temp)+(1.0-alpha)*t3Valued[r-1][1]*t3Valued[r-1][1]);
          temp = iT3(t3Valued[r][1],period,i,12); t3Valued[r][2] = MathSqrt(alpha*(t3Valued[r][1]-temp)*(t3Valued[r][1]-temp)+(1.0-alpha)*t3Valued[r-1][2]*t3Valued[r-1][2]);
          temp = iT3(t3Valued[r][2],period,i,18); t3Valued[r][3] = MathSqrt(alpha*(t3Valued[r][2]-temp)*(t3Valued[r][2]-temp)+(1.0-alpha)*t3Valued[r-1][3]*t3Valued[r-1][3]);
          temp = iT3(t3Valued[r][3],period,i,24); t3Valued[r][4] = MathSqrt(alpha*(t3Valued[r][3]-temp)*(t3Valued[r][3]-temp)+(1.0-alpha)*t3Valued[r-1][4]*t3Valued[r-1][4]);
          temp = iT3(t3Valued[r][4],period,i,30); t3Valued[r][5] = MathSqrt(alpha*(t3Valued[r][4]-temp)*(t3Valued[r][4]-temp)+(1.0-alpha)*t3Valued[r-1][5]*t3Valued[r-1][5]);
   return(MathSqrt(n)*(c1*t3Valued[r][5] + c2*t3Valued[r][4] + c3*t3Valued[r][3] + c4*t3Valued[r][2]));
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}